{
Copyright  1998 by Delphi 4 Developer's Guide - Xavier Pacheco and Steve Teixeira
}
unit DBSound;

interface

uses Windows, Messages, Classes, SysUtils, Controls, Buttons, DB, DBTables,
  DbCtrls;

type
  EDBWavError = class(Exception);

  TDBWavPlayer = class(TSpeedButton)
  private
    FAutoPlay: Boolean;
    FDataLink: TFieldDataLink;
    FDataStream: TMemoryStream;
    FExceptOnError: Boolean;
    procedure DataChange(Sender: TObject);
    function GetDataField: string;
    function GetDataSource: TDataSource;
    function GetField: TField;
    procedure SetDataField(const Value: string);
    procedure SetDataSource(Value: TDataSource);
    procedure CMGetDataLink(var Message: TMessage); message CM_GETDATALINK;
    procedure CreateDataStream;
    procedure PlaySound;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Click; override;
    property Field: TField read GetField;
  published
    property AutoPlay: Boolean read FAutoPlay write FAutoPlay default False;
    property ExceptOnError: Boolean read FExceptOnError write FExceptOnError;
    property DataField: string read GetDataField write SetDataField;
    property DataSource: TDataSource read GetDataSource write SetDataSource;
  end;

implementation

uses MMSystem;

const
  // Error strings
  SNotBlobField = 'Field "%s" is not a blob field';
  SPlaySoundErr = 'Error attempting to play sound';

constructor TDBWavPlayer.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);             // call inherited
  FDataLink := TFieldDataLink.Create;   // create field data link
  FDataLink.OnDataChange := DataChange; // get data link notifications
  FDataStream := TMemoryStream.Create;  // create worker memory stream
end;

destructor TDBWavPlayer.Destroy;
begin
  FDataStream.Free;
  FDataLink.Free;
  FDataLink := Nil;
  inherited Destroy;
end;

procedure TDBWavPlayer.Click;
begin
  inherited Click; // do default behavior
  PlaySound;       // play the sound
end;

procedure TDBWavPlayer.CreateDataStream;
// creates memory stream from wave file in blob field
var
  BS: TBlobStream;
begin
  // make sure it's a blob field
  if not (Field is TBlobField) then
    raise EDBWavError.CreateFmt(SNotBlobField, [DataField]);
  // create a blob stream
  BS := TBlobStream.Create(TBlobField(Field), bmRead);
  try
    // copy from blob stream to memory stream
    FDataStream.SetSize(BS.Size);
    FDataStream.CopyFrom(BS, BS.Size);
  finally
    BS.Free;  // free blob stream
  end;
end;

procedure TDBWavPlayer.PlaySound;
// plays wave sound loaded in memory stream
begin
  // make sure we are hooked to a dataset and field
  if (DataSource <> nil) and (DataField <> '') then
  begin
    // make sure data stream is created
    if FDataStream.Size = 0 then CreateDataStream;
    // Play the sound in the memory stream, raise exception on error
    if (not MMSystem.PlaySound(FDataStream.Memory, 0, SND_ASYNC or SND_MEMORY))
      and FExceptOnError then
      raise EDBWavError.Create(SPlaySoundErr);
  end;
end;

procedure TDBWavPlayer.DataChange(Sender: TObject);
// OnChange handler FFieldDataLink.DataChange
begin
  // deallocate memory occupied by previous wave file
  with FDataStream do if Size <> 0 then SetSize(0);
  // if AutoPlay is on, the play the sound
  if FAutoPlay then PlaySound;
end;

procedure TDBWavPlayer.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  // do some required housekeeping
  if (Operation = opRemove) and (FDataLink <> nil) and
    (AComponent = DataSource) then DataSource := nil;
end;

function TDBWavPlayer.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

procedure TDBWavPlayer.SetDataSource(Value: TDataSource);
begin
  FDataLink.DataSource := Value;
  if Value <> nil then Value.FreeNotification(Self);
end;

function TDBWavPlayer.GetDataField: string;
begin
  Result := FDataLink.FieldName;
end;

procedure TDBWavPlayer.SetDataField(const Value: string);
begin
  FDataLink.FieldName := Value;
end;

function TDBWavPlayer.GetField: TField;
begin
  Result := FDataLink.Field;
end;

procedure TDBWavPlayer.CMGetDataLink(var Message: TMessage);
begin
  Message.Result := Integer(FDataLink);
end;

end.

