unit TblUtils;

interface

uses DB, DBTables, Classes;

function FieldTypeDelphiToSQL(DelType: TFieldType): string;
procedure FieldTypeDelphiToBDE(DelType: TFieldType; var BDEType, BDESubType: word);
function SumField(T: TTable; Field: String): Extended;
function AvgField(T: TTable; Field: String): Extended;
function MinField(T: TTable; Field: String): Extended;
function MaxField(T: TTable; Field: string): Extended;
procedure DropFieldFromSQLTable(T: TTable; FieldName: string);
procedure AddFieldToSQLTable(T: TTable; FieldType: TFieldType; FieldName: string;
  FieldSize, Precision: Integer);
procedure DropFieldFromLocalTable(T: TTable; FieldName: string);
procedure AddFieldToLocalTable(T: TTable; FieldType: TFieldType; FieldName: string;
  FieldSize, Precision: Integer);
procedure QuickCopyTable(T: TTable; DestTblName: string; Overwrite: Boolean);
procedure GetPDoxUsersForSession(Sess: TSession; UserList: TStrings);

implementation

uses SysUtils, BDE, Dialogs;
{$J-}

type
  TSQLAggFunc = (safSum, safAvg, safMin, safMax);

const
  // SQL aggregate functions
  SQLAggStrs: array[TSQLAggFunc] of string = (
    'select sum(%s) from %s',
    'select avg(%s) from %s',
    'select min(%s) from %s',
    'select max(%s) from %s');

  // SQL alter table functions to add and drop fields
  SQLAddStr = 'alter table "%s" add %s %s';
  SQLDropStr = 'alter table "%s" drop %s';

function CreateQueryFromTable(T: TTable): TQuery;
// returns a query hooked to the same database and session as the give table
begin
  Result := TQuery.Create(nil);
  try
    Result.DatabaseName := T.DatabaseName;
    Result.SessionName := T.SessionName;
  except
    Result.Free;
    Raise;
  end;
end;

function FieldTypeDelphiToSQL(DelType: TFieldType): string;
// maps a Delphi TFieldType variable to a SQL data type.
// %d formatter is left where byte-size information should go.
// %s formatter is left where decimal size, precision info should go.
begin
  case DelType of
    ftString: Result := 'CHAR(%d)';
    ftSmallint: Result := 'SMALLINT';
    ftInteger: Result := 'INTEGER';
    ftWord: Result := 'SMALLINT';
    ftBoolean: Result := 'BOOLEAN';
    ftFloat: Result := 'FLOAT(%s)';
    ftCurrency: Result := 'MONEY';
    ftBCD: Result := 'DECIMAL(%s)';
    ftDate: Result := 'DATE';
    ftTime: Result := 'TIME';
    ftDateTime: Result := 'TIMESTAMP';
    ftBytes: Result := 'BYTES(%d)';
    ftAutoInc: Result := 'AUTOINC';
    ftBlob: Result := 'BLOB(%d, 2)';
    ftMemo: Result := 'BLOB(%d, 1)';
    ftGraphic: Result := 'BLOB(%d, 5)';
    ftFmtMemo: Result := 'BLOB(%d, 3)';
    ftParadoxOle: Result := 'BLOB(%d, 4)';
    // if we don't know what it is, use integer
  else  Result := 'INTEGER';
  end;
end;

procedure FieldTypeDelphiToBDE(DelType: TFieldType; var BDEType, BDESubType: word);
// maps a Delphi TFieldType to a BDE field type and subtype
begin
  BDEType := fldUnknown;
  BDESubType := fldUnknown;
  case DelType of
    ftString: BDEType := fldZSTRING;
    ftSmallint: BDEType := fldINT16;
    ftInteger: BDEType := fldINT32;
    ftWord: BDEType := fldUINT16;
    ftBoolean: BDEType := fldBOOL;
    ftFloat: BDEType := fldFLOAT;
    ftCurrency:
      begin
        BDEType := fldFLOAT;
        BDESubType := fldstMONEY;
      end;
    ftBCD: BDEType := fldBCD;
    ftDate: BDEType := fldDATE;
    ftTime: BDEType := fldTIME;
    ftDateTime: BDEType := fldTIMESTAMP;
    ftBytes: BDEType := fldBYTES;
    ftVarBytes: BDEType := fldVARBYTES;
    ftAutoInc:
      begin
        BDEType := fldINT32;
        BDESubType := fldstAUTOINC;
      end;
    ftBlob:
      begin
        BDEType := fldBLOB;
        BDESubType := fldstBINARY;
      end;
    ftMemo:
      begin
        BDEType := fldBLOB;
        BDESubType := fldstMEMO;
      end;
    ftGraphic:
      begin
        BDEType := fldBLOB;
        BDESubType := fldstGRAPHIC;
      end;
    ftFmtMemo:
      begin
        BDEType := fldBLOB;
        BDESubType := fldstFMTMEMO;
      end;
    ftParadoxOle:
      begin
        BDEType := fldBLOB;
        BDESubType := fldstOLEOBJ;
      end;
    ftDBaseOle:
      begin
        BDEType := fldBLOB;
        BDESubType := fldstDBSOLEOBJ;
      end;
    ftTypedBinary:
      begin
        BDEType := fldBLOB;
        BDESubType := fldstTYPEDBINARY;
      end;
  end;
end;

function DoSQLAggFunc(T: TTable; FieldNames: string;
  Func: TSQLAggFunc): Extended;
begin
  with CreateQueryFromTable(T) do
  begin
    try
      SQL.Add(Format(SQLAggStrs[Func], [FieldNames, T.TableName]));
      Open;
      Result := Fields[0].AsFloat;
    finally
      Free;
    end;
  end;
end;

function SumField(T: TTable; Field: String): Extended;
begin
  Result := DoSQLAggFunc(T, Field, safSum);
end;

function AvgField(T: TTable; Field: String): Extended;
begin
  Result := DoSQLAggFunc(T, Field, safAvg);
end;

function MinField(T: TTable; Field: String): Extended;
begin
  Result := DoSQLAggFunc(T, Field, safMin);
end;

function MaxField(T: TTable; Field: string): Extended;
begin
  Result := DoSQLAggFunc(T, Field, safMax);
end;

procedure DropFieldFromSQLTable(T: TTable; FieldName: string);
// uses SQL to drop a field from a local or remote table
var
  WasActive: Boolean;
begin
  WasActive := T.Active;
  if WasActive then T.Close;
  with CreateQueryFromTable(T) do
  begin
    try
      SQL.Add(Format(SQLDropStr, [T.TableName, FieldName]));
      ExecSQL;
    finally
      Free;
      T.Active := WasActive;
    end;
  end;
end;

procedure AddFieldToSQLTable(T: TTable; FieldType: TFieldType;
  FieldName: string; FieldSize, Precision: Integer);
// uses SQL to add a field to a local or remote table
// if Precision = -1 then uses default
var
  FldStr: string;
  WasActive: Boolean;
begin
  // map field to SQL field string
  FldStr := FieldTypeDelphiToSQL(FieldType);
  case FieldType of
    ftString, ftBytes, ftBlob, ftMemo, ftGraphic, ftFmtMemo, ftParadoxOle:
      FldStr := Format(FldStr, [FieldSize]);
    ftFloat, ftBCD:
      if Precision = -1 then FldStr := Format(FldStr, [IntToStr(FieldSize)])
      else FldStr := Format(FldStr, [IntToStr(FieldSize) + ',' + IntToStr(Precision)]);
  end;
  // create and execute the query
  WasActive := T.Active;
  if WasActive then T.Close;
  with CreateQueryFromTable(T) do
  begin
    try
      SQL.Add(Format(SQLAddStr, [T.TableName, FieldName, FldStr]));
      ExecSQL;
    finally
      Free;
      T.Active := WasActive;
    end;
  end;
end;

procedure DropFieldFromLocalTable(T: TTable; FieldName: string);
// uses DbiDoRestructure() to drop a field from a local table
var
  PFD, HeadFD, TailFD: pFldDesc;
  FD: FldDesc;
  TD: CRTblDesc;
  POT, TempOT: pCROpType;
  WasOpen: Boolean;
  NumCopied: word;
  TempHDB: hDBIDB;
  NumFlds, i, FldNumToDrop: Integer;
  DBType: DBINAME;
begin
  WasOpen := T.Active;        // save table active state
  if not WasOpen then T.Open; // ensure table is open
  NumFlds := T.FieldCount;    // save table field count
  // get persistant database handle from table handle
  Check(DbiGetObjFromObj(hDBIObj(T.Handle), objDATABASE, hDBIObj(TempHDB)));
  // allocate memory for field desc array and field ops array
  PFD := AllocMem(NumFlds * SizeOf(FldDesc));
  POT := AllocMem(NumFlds * SizeOf(CROpType));
  try
    if NumFlds = 0 then raise EDatabaseError.Create('Table has no fields');
    Check(DbiGetFieldDescs(T.Handle, PFD));  // get field descs
    if WasOpen then T.Close;                 // close table
    FldNumToDrop := -1;
    // iterate over fields and find number of field to drop
    // mark bad field for delete, rest for noop
    HeadFD := PFD;
    TempOT := POT;
    // initialize field op array with crNOOP
    for i := 1 to NumFlds do
    begin
      TempOT^ := crNOOP;
      inc(TempOT);
    end;
    // look for number of field to drop by name
    for i := 1 to NumFlds do
    begin
      if StrIComp(HeadFD^.szName, PChar(FieldName)) = 0 then
        FldNumToDrop := i;
      inc(HeadFD);
    end;
    // raise an exception if field not found
    if FldNumToDrop = -1 then
      raise EDatabaseError.CreateFmt('Field %s does not exist in table',
        [FieldName]);
    // if we are dropping a field from the middle of the table...
    if FldNumToDrop <> NumFlds then
    begin
      HeadFD := PFD;
      Inc(HeadFD, FldNumToDrop - 1);   // get pointer to field to drop
      FD := HeadFD^;                   // save field desc
      TailFD := HeadFD;
      Inc(TailFD);                     // get pointer to next field
      // shift all fields after the one to drop to the left
      Move(TailFD^, HeadFD^, SizeOf(FldDesc) * (NumFlds - FldNumToDrop));
      Inc(TailFD, (NumFlds - FldNumToDrop - 1));  // point to end of array
      TailFD^ := FD;                   // add field to drop to end
    end;
    // since field to drop will be the last in the array at this point,
    // we can set the corresponding op type to crDROP
    TempOT := POT;
    Inc(TempOT, NumFlds - 1);
    TempOT^ := crDROP;

    // debug stuff
    HeadFD := PFD;
    TempOT := POT;
    for i := 1 to NumFlds do
    begin
      HeadFD^.iFldNum := i;
      ShowMessage(Format('Field is: %s.  Number is %d.  POT is %d.',
        [HeadFD^.szName, HeadFD^.iFldNum, Ord(TempOT^)]));
      inc(HeadFD);
      inc(TempOT);
    end;
    // Get database type
    Check(DbiGetProp(hDBIObj(TempHDB), dbDATABASETYPE, @DBType, SizeOf(DBINAME), NumCopied));
    // Create new table desc
    FillChar(TD, SizeOf(TD), 0);
    with TD do
    begin
      StrPCopy(szTblName, T.TableName);
      StrCopy(szDbType, DBType);
      iFldCount := NumFlds - 1;
      pFldDesc := PFD;
      pecrFldOp := POT;
    end;
    Check(DbiDoRestructure(TempHDB, 1, @TD, nil, nil, nil, False));
  finally
    FreeMem(POT, NumFlds * SizeOf(CROpType));
    FreeMem(PFD, NumFlds * SizeOf(FldDesc));
    T.Active := WasOpen;
  end;
end;

procedure AddFieldToLocalTable(T: TTable; FieldType: TFieldType; FieldName: string;
  FieldSize, Precision: Integer);
// uses DbiDoRestructure() to add a field to a local table
var
  PFD: pFldDesc;
  TD: CRTblDesc;
  POT: pCROpType;
  WasOpen: Boolean;
  FldType, FldSubType, NumCopied: word;
  TempHDB: hDBIDB;
  NumFlds, i: Integer;
  DBType: DBINAME;
begin
  WasOpen := T.Active;        // save table active state
  if not WasOpen then T.Open; // ensure table is open
  NumFlds := T.FieldCount;    // save table field count
  // get persistant database handle from table handle
  Check(DbiGetObjFromObj(hDBIObj(T.Handle), objDATABASE, hDBIObj(TempHDB)));
  // allocate memory for field desc array and field ops array
  PFD := AllocMem((NumFlds + 1) * SizeOf(FldDesc));
  POT := AllocMem((NumFlds + 1) * SizeOf(CROpType));
  try
    Check(DbiGetFieldDescs(T.Handle, PFD));  // get field descs
    if WasOpen then T.Close;                 // close table
    inc(PFD, NumFlds);                       // inc to last field desc
    // fill in field desc for new field
    with PFD^ do
    begin
      iFldNum := NumFlds + 1;
      StrPCopy(szName, FieldName);
      FieldTypeDelphiToBDE(FieldType, FldType, FldSubType);
      iFldType := FldType;
      iSubType := FldSubType;
      iUnits1 := FieldSize;
      iUnits2 := Precision;
    end;
    Dec(PFD, NuMFlds);
    // don't modify existing fields
    for i := 1 to NumFlds do
    begin
      POT^ := crNOOP;
      inc(POT);
    end;
    POT^ := crAdd;                           // add new field
    Dec(POT, NumFlds);
    // Get database type
    Check(DbiGetProp(hDBIObj(TempHDB), dbDATABASETYPE, @DBType,
      SizeOf(DBINAME), NumCopied));
    // Create new table desc
    FillChar(TD, SizeOf(TD), 0);
    with TD do
    begin
      StrPCopy(szTblName, T.TableName);
      StrCopy(szDbType, DBType);
      iFldCount := NumFlds + 1;
      pFldDesc := PFD;
      pecrFldOp := POT;
    end;
    Check(DbiDoRestructure(TempHDB, 1, @TD, Nil, Nil, Nil, False));
  finally
    FreeMem(POT, (NumFlds + 1) * SizeOf(CROpType));
    FreeMem(PFD, (NumFlds + 1) * SizeOf(FldDesc));
    T.Active := WasOpen;
  end;
end;

procedure QuickCopyTable(T: TTable; DestTblName: string; Overwrite: Boolean);
// Copies TTable T to an identical table with name DestTblName.
// Will overwrite existing table with name DestTblName if Overwrite is True.
var
  DBType: DBINAME;
  WasOpen: Boolean;
  NumCopied: Word;
begin
  WasOpen := T.Active;        // save table active state
  if not WasOpen then T.Open; // ensure table is open
  // Get driver type string
  Check(DbiGetProp(hDBIObj(T.Handle), drvDRIVERTYPE, @DBType,
    SizeOf(DBINAME), NumCopied));
  // Copy the table
  Check(DbiCopyTable(T.DBHandle, Overwrite, PChar(T.TableName), DBType,
    PChar(DestTblName)));
  T.Active := WasOpen;        // restore active state
end;

procedure GetPDoxUsersForSession(Sess: TSession; UserList: TStrings);
// Clears UserList and adds each user using the same netfile as session
// Sess to the list.  If Sess = nil, then procedure works for default
// net file.
var
  WasActive: Boolean;
  SessHand: hDBISes;
  ListCur: hDBICur;
  User: UserDesc;
begin
  if UserList = nil then Exit;
  WasActive := False;
  UserList.Clear;
  if Assigned(Sess) then
  begin
    WasActive := Sess.Active;
    if not WasActive then Sess.Open;
    Check(DbiStartSession(Nil, SessHand, PChar(Sess.NetFileDir)));
  end
  else
    Check(DbiStartSession(Nil, SessHand, Nil));
  try
    Check(DbiOpenUserList(ListCur));
    while DbiGetNextRecord(ListCur, dbiNOLOCK, @User, Nil) = DBIERR_NONE do
      UserList.Add(User.szUserName);
  finally
    DbiCloseSession(SessHand);
    if Assigned(Sess) then Sess.Active := WasActive;
  end;
end;

initialization
  DbiInit(Nil);
finalization
  DbiExit;
end.
