unit DDGUtils;

interface

uses Windows, SysUtils, Classes;

type
  EExecuteError = class(Exception);
  ERegError = class(Exception);

{ ComparePtr compare the address of two pointers.  It returns -1 if the    }
{ address of P1 is lower than the address of P2, 0 if the address of P1 is }
{ equal to the address of P2, or 1 if the address of P1 is higher than P2. }
function ComparePtr(P1, P2: Pointer): longint;

// CreateCapturedOutputProcess executes the program specified by
// CommandLine, and pipes the output generated by that program into Buffer.
function CreateCapturedOutputProcess(const CommandLine: string; var Buffer;
  BufSize: Integer; var BytesRead: Integer): DWORD;

// CreateProcessAndWait executes program given by AppPath and
// suspends the current thread until AppPath terminates
function CreateProcessAndWait(const AppPath, AppParams: String;
                              Visibility: word): DWord;

// GetFileExt returns the file extension of FileName
function GetFileExt(const FileName: String): String;

// StreamEqual compares memory streams S1 and S2 and returns
// True if they are equal.
function StreamsEqual(S1, S2: TMemoryStream): Boolean;

// WinExecAndWait executes program given by Path and suspends
// the current thread until Path terminates
function WinExecAndWait(Path: String; Visibility: word): DWord;

implementation

uses StrUtils;

function ComparePtr(P1, P2: Pointer): longint;
{ -> eax   P1             }
{ -> edx   P2             }
{ <- eax   -1 if P1 < P2  }
{           0 if P1 = P2  }
{           1 if P1 > P2  }
asm
  cmp eax, edx  { compare pointers }
  jge @@1       { jump if eax >= edx }
  mov eax, -1   { return less than }
  jmp @@3
@@1:
  cmp eax, edx  { compare pointers }
  jg @@2        { jump if eax > edx }
  mov eax, 0    { return equal to }
  jmp @@3
@@2:
  mov eax, 1    { return greater than }
@@3:
end;

function CreateCapturedOutputProcess(const CommandLine: string; var Buffer;
  BufSize: Integer; var BytesRead: Integer): DWORD;
var
  SI: TStartupInfo;
  PI: TProcessInformation;
  OutHand, PipeRead, PipeWrite: THandle;
  BuffPtr: PByte;
  SA: TSecurityAttributes;
begin
  OutHand := GetStdHandle(STD_OUTPUT_HANDLE);  // Save current output handle
  try
    with SA do
    begin
      nLength := SizeOf(SA);
      lpSecurityDescriptor := Nil;
      bInheritHandle := True;                  // make handles inheritable
    end;
    // Create anonymous pipe, and use it as the output handle
    Win32Check(CreatePipe(PipeRead, PipeWrite, @SA, 0));
    Win32Check(SetStdHandle(STD_OUTPUT_HANDLE, PipeWrite));
    FillChar(SI, SizeOf(SI), 0);
    with SI do
    begin
      cb := SizeOf(SI);
      dwFlags := STARTF_USESHOWWINDOW;
      wShowWindow := SW_HIDE;                  // don't show window
    end;
    try
      // Create the process, child process inherits our handles
      Win32Check(CreateProcess(Nil, PChar(CommandLine), Nil, Nil, True, 0, Nil,
        Nil, SI, PI));
    except
      // if an error occurs while creating the process, make sure pipe
      // handles are closed.
      CloseHandle(PipeRead);
      CloseHandle(PipeWrite);
      raise;
    end;
    CloseHandle(PI.hThread);   // close child process' thread handle
    CloseHandle(PipeWrite);    // close pipe write handle
    BuffPtr := @Buffer;
    BytesRead := 0;
    // keep reading from pipe until we run out of buffer or until ReadFile
    // returns False.  It will return False when child process closes it's
    // write handle for pipe.
    while (BytesRead < BufSize) and ReadFile(PipeRead, BuffPtr^,
      BufSize - BytesRead, BytesRead, Nil) do inc(BuffPtr, BytesRead);
    CloseHandle(PipeRead);                       // done with read handle
    WaitForSingleObject(PI.hProcess, INFINITE);  // make sure process terminates
    GetExitCodeProcess(PI.hProcess, Result);     // Get exit code for process
    CloseHandle(PI.hProcess);                    // close child process handle
  finally
    SetStdHandle(STD_OUTPUT_HANDLE, OutHand);    // restore original output handle
  end;
end;

function CreateProcessAndWait(const AppPath, AppParams: String;
                              Visibility: word): DWord;
var
  SI: TStartupInfo;
  PI: TProcessInformation;
  Proc: THandle;
begin
  FillChar(SI, SizeOf(SI), 0);
  SI.cb := SizeOf(SI);
  SI.wShowWindow := Visibility;
  if not CreateProcess(PChar(AppPath), PChar(AppParams), Nil, Nil, False,
                   Normal_Priority_Class, Nil, Nil, SI, PI) then
    raise EExecuteError.CreateFmt('Failed to execute program.  Error Code %d',
                                  [GetLastError]);
  Proc := PI.hProcess;
  CloseHandle(PI.hThread);
  if WaitForSingleObject(Proc, Infinite) <> Wait_Failed then
    GetExitCodeProcess(Proc, Result);
  CloseHandle(Proc);
end;

function GetFileExt(const FileName: String): String;
var
  ExtPos: integer;
begin
  Result := ExtractFilename(FileName);     // strip directory info
  ExtPos := LastPos('.', Result);          // get last dot in filename
  { Return extension if '.' is found, otherwise return '' }
  if ExtPos = 0 then
    Result := ''
  else
    Result := Copy(Result, ExtPos + 1, Length(Result));
end;

function StreamsEqual(S1, S2: TMemoryStream): Boolean;
begin
  Result := (S1.Size = S2.Size) and CompareMem(S1.Memory, S2.Memory, S1.Size);
end;

function WinExecAndWait(Path: String; Visibility: word): DWord;
var
  P: integer;
  Params: String;
begin
  P := Pos(' ', Path);   // assume params start at first space
  Params := Copy(Path, P, Length(Path));
  Delete(Path, P, Length(Path));
  Result := CreateProcessAndWait(Path, Params, Visibility);
end;

end.
